// Text conversion from WordStar 4.0 CP/M to RTF
// WordStar text in default Windows encoding
// P 2018

using System;
using System.IO;
using System.Text;

static class ws2rtf
{
	public static void Main(string[] args)
	{
		if (args.Length < 2) {
			Console.WriteLine("Usage: ws2rtf WSfile RTFfile");
			Environment.Exit(1);
		}

		const byte softCR = 0x8d; // soft end of line
		var input = new StreamReader(args[0], Encoding.Default);
		var output = new StreamWriter(args[1], false, Encoding.ASCII);
		string line;
		bool bUnder  = false;
		bool bBold   = false;
		bool bItal   = false;
		bool bSuper  = false;
		bool bSub    = false;
		bool bStrike = false;
		bool bDouble = false;

		// RTF header
		output.Write('{');
		output.WriteLine(@"\rtf1\ansi\ansicpg{0}\deff1", Encoding.Default.CodePage);
		output.WriteLine(@"{\fonttbl{\f0\fnil default;}{\f1\froman Times New Roman;}}");

		while ((line = input.ReadLine()) != null) {
			// empty lines
			if (string.IsNullOrEmpty(line)) {
				output.WriteLine(@"\par");
				continue;
			}
			// skip over dot commands
			if (line[0] == '.') {
				continue;
			}
			byte[] inputBytes = Encoding.Default.GetBytes(line);
			int bytes = inputBytes.Length - 1;
			if (inputBytes[bytes] != softCR) {
				bytes++;
			}
			for (int i = 0; i < bytes; i++) {
				byte b = inputBytes[i];
				switch (b) {
				  case 0x02: // bold
					  if (bBold) {
							output.Write(@"\b0 ");
						} else {
							output.Write(@"\b ");
						}
						bBold = !bBold;
					  continue;
				  case 0x04: // double width font, emulated by inserting non-breaking spaces
						bDouble = !bDouble;
					  continue;
					case 0x09: // tab
						output.Write(@"\tab ");
						continue;
					case 0x0c: // page break
						output.WriteLine(@"\page");
						continue;
					case 0x0f: // non-breaking space
						output.Write(@"\~");
						continue;
				  case 0x13: // underlined
					  if (bUnder) {
							output.Write(@"\ul0 ");
						} else {
							output.Write(@"\ul ");
						}
						bUnder = !bUnder;
					  continue;
				  case 0x14: // superscript
					  if (bSuper) {
							output.Write(@"\nosupersub ");
						} else {
							output.Write(@"\super ");
						}
						bSuper = !bSuper;
					  continue;
				  case 0x16: // subscript
					  if (bSub) {
							output.Write(@"\nosupersub ");
						} else {
							output.Write(@"\sub ");
						}
						bSub = !bSub;
					  continue;
				  case 0x18: // strike
					  if (bStrike) {
							output.Write(@"\strike0 ");
						} else {
							output.Write(@"\strike ");
						}
						bStrike = !bStrike;
					  continue;
				  case 0x19: // italics
					  if (bItal) {
							output.Write(@"\i0 ");
						} else {
							output.Write(@"\i ");
						}
						bItal = !bItal;
					  continue;
					case 0x1c: // soft hyphen
						output.Write(@"\-");
						continue;
					case 0xa0: // formatting space
						continue;
				}
				if (b >= 0x20 && b <= 0x7e) {
					if (b == '\\' || b == '{' || b == '}') {
						output.Write('\\');
					}
					output.Write(Convert.ToChar(b));
					if (bDouble) {
						output.Write(@"\~");
					}
				}
				if (b >= 0x80) {
					output.Write(@"\'{0:X2}", b);
					if (bDouble) {
						output.Write(@"\~");
					}
				}
			} // for - line loop
			if (inputBytes[inputBytes.Length - 1] != softCR) {
				output.Write(@"\par");
			}
			output.WriteLine();
		} // while - file reading loop

		output.Write('}');
		output.Close();
		input.Close();
  }
}
